﻿//Matt Monasch: mpm1052@rit.edu
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using BaalaAgni.Collisions;
using System.Diagnostics;

namespace BaalaAgni
{
    using Load;
    using Effects;
    public class Skill
    {
        
        private string skillName;//Skill needs a name
        public string SkillName
        {
            get { return skillName; }
            set { skillName = value; }
        }

        private int power;//Skill will have a certain amount of power
        public int Power
        {
            get { return power; }
            set { power = value; }
        }

        private double coolDown;//Skill will have a cooldown, even if that cooldown may be 0. This is in milliseconds
        public double CoolDown
        {
            get { return coolDown; }
            set { coolDown = value; }
        }

        private bool stationary; //dictates whether or not the skill should move. For instance, a lava pit won't move but a fireball will.
        public bool Stationary
        {
            get { return stationary; }
            set { stationary = value; }
        }

        private float speed;//dictates how fast the skill will move (granted that it isn't stationary). Default is 1 which is simply a unit vector. The number will be a scalar for the moveEffect
        public float Speed
        {
            get { return speed; }
            set { speed = value; }
        }

        private bool hasSkill;//The skill itself knows if the Warlock hast the skill or not
        public bool HasSkill
        {
            get { return hasSkill; }
            set { hasSkill = value; }
        }

        private Stopwatch timer;//This will keep track of the time since last cast to enforce the cooldown
        public Stopwatch Timer
        {
            get { return timer; }
            set { timer = value; }
        }

        public static List<Skill> SkillList = new List<Skill>();

        public Skill()
        {
            skillName = "fireball";
            power = 20;
            coolDown = 500;
            speed = 3;
            stationary = false;
            timer = new Stopwatch();
            hasSkill = true;
            SkillList.Add(this);
        }

        /// <summary>
        /// Create the skill as a projectile and have it act as directed by its fields
        /// </summary>
        private void performSkill(Vector2 mousePos)
        {
            if (hasSkill)
            {
                if (!timer.IsRunning || timer.ElapsedMilliseconds > coolDown)//if more time has elapsed than the cooldown number is, then cast again
                {
                    Music.GetInstance().PlaySkillEffect(0);
                    Character warlock = BaalaAgni.getInstance().Model.PC;//get an easier reference to the Warlock
                    Projectile proj = new Projectile();
                    proj.ShotFrom = warlock;
                    proj.Name = this.skillName;//set the name
                    proj.Faction = BaalaAgni.getInstance().Model.PC.Faction;
                    proj.CollisionEff = new CollisionEffect(skillName + "collide", this.power);
                    proj.SceneNode = Storage.Scenes[skillName].copy();//set the sceneNode
                    proj.SceneNode.Position = new Vector2(((Rectangle)BaalaAgni.getInstance().Model.PC.CollisionMap).Center.X, ((Rectangle)BaalaAgni.getInstance().Model.PC.CollisionMap).Center.Y);//set its position to be the center of the Warlock

                    if (stationary)
                    {
                        proj.MoveEff.Direction = new Vector2(0, 0);
                        proj.SceneNode.Position = new Vector2(mousePos.X + BaalaAgni.getInstance().View.Camera.Position.X - (proj.CollisionMap.Size.X / 2), mousePos.Y + BaalaAgni.getInstance().View.Camera.Position.Y - (proj.CollisionMap.Size.Y / 2));//if its stationary, then its position is where the user clicked
                    }
                    else
                    {
                        Vector2 dir = new Vector2((float)(mousePos.X + BaalaAgni.getInstance().View.Camera.Position.X - (((Rectangle)warlock.CollisionMap).Center.X)) - (proj.CollisionMap.Size.X / 2), (float)(mousePos.Y + BaalaAgni.getInstance().View.Camera.Position.Y - (((Rectangle)warlock.CollisionMap).Center.Y) - (proj.CollisionMap.Size.Y / 2)));//get a vector that will send it to the clicked point
                        dir.Normalize();//make it a unit vector
                        proj.MoveEff = new MoveEffect(skillName + "move", dir * speed);//set the moveeffect so that it has its unitvector multiplied by its intended speed.
                    }

                    timer.Reset();//restart the cooldown timer and then start it again
                    timer.Start();
                    if (proj.SceneNode.containsTexture("start"))
                    {
                        proj.SceneNode.playOnce("start", "mid");
                    }
                    BaalaAgni.getInstance().Model.Region.addEntity(proj);
                }
            }
        }
        /// <summary>
        /// Use this and pass it the current Skill number and Skill will do the rest.
        /// </summary>
        public static void PerformSkill(int skillNum, Vector2 mousePos)//gets the correct position for the skill and calls its performEffect
        {
            SkillList[skillNum-1].performSkill(mousePos);
            
        }
        
        public static void PerformBossSkill(Character boss)//boss monsters call this skill so that they can fight the warlock
        {
            Stopwatch bossTimer = boss.bossSkillTimer;
            
            Character warlock = BaalaAgni.getInstance().Model.PC;//get an easier reference to the Warlock
            Projectile proj = new Projectile();
            Random rand = new Random();
            switch (boss.Name)
            {
                case "fire":
                    
                    if (bossTimer.ElapsedMilliseconds > 3000 && rand.Next(30) == 15)
                    {
                        Music.GetInstance().PlaySkillEffect(0);
                        bossTimer.Reset();
                        bossTimer.Start();
                        proj.Name = "fireball";//set the name
                        proj.Faction = boss.Faction;//set faction equal to what shot it
                        proj.CollisionEff = new CollisionEffect("fireboss" + "collide", 500);
                        proj.SceneNode = Storage.Scenes["fireball"].copy();//set the sceneNode
                        proj.SceneNode.Position = new Vector2(((Rectangle)boss.CollisionMap).Center.X, ((Rectangle)boss.CollisionMap).Center.Y);
                        Vector2 dir = new Vector2((float)(((Rectangle)warlock.CollisionMap).Center.X - (((Rectangle)boss.CollisionMap).Center.X)) - (proj.CollisionMap.Size.X / 2), (float)(((Rectangle)warlock.CollisionMap).Center.Y - (((Rectangle)boss.CollisionMap).Center.Y)) - (proj.CollisionMap.Size.Y / 2));//get a vector that will send it to the Warlock
                        dir.Normalize();//make it a unit vector
                        proj.MoveEff = new MoveEffect("fireball" + "move", dir);//set the moveeffect so that it has its unitvector multiplied by its intended speed.

                        if (proj.SceneNode.containsTexture("start"))//this is how the animation shows itself in its separate stages
                        {
                            proj.SceneNode.playOnce("start", "mid");
                        }
                        BaalaAgni.getInstance().Model.Region.addEntity(proj);
                    }
                    else if (!bossTimer.IsRunning)
                    {
                        bossTimer.Start();
                    }
                    break;
                case "ice":
                    if (bossTimer.ElapsedMilliseconds > 2500)
                    {
                        Music.GetInstance().PlaySkillEffect(0);
                        bossTimer.Reset();
                        bossTimer.Start();
                        proj.Name = "ice";
                        proj.ShotFrom = boss;
                        proj.Faction = boss.Faction;
                        proj.CollisionEff = new CollisionEffect("iceboss" + "collide", 500);
                        proj.SceneNode = Storage.Scenes["ice"].copy();
                        proj.SceneNode.Position = new Vector2(((Rectangle)warlock.CollisionMap).Center.X + boss.MoveEff.Direction.X * 1500 - ((Rectangle)proj.CollisionMap).Width/2, ((Rectangle)warlock.CollisionMap).Center.Y + boss.MoveEff.Direction.Y * 1500 - ((Rectangle)proj.CollisionMap).Height/2);
                        proj.MoveEff = new MoveEffect("icefield" + "move", new Vector2(0, 0));
                        if (proj.SceneNode.containsTexture("start"))//this is how the animation shows itself in its separate stages
                        {
                            proj.SceneNode.playOnce("start", "mid");
                        }
                        BaalaAgni.getInstance().Model.Region.addEntity(proj);
                    }
                    else if (!bossTimer.IsRunning)
                    {
                        bossTimer.Start();
                    }
                    break;
                case "thunder":
                    if (bossTimer.ElapsedMilliseconds > 5000)
                    {
                        Music.GetInstance().PlaySkillEffect(0);
                        bossTimer.Reset();
                        bossTimer.Start();
                        proj.Name = "thundercloud";
                        proj.ShotFrom = boss;
                        proj.Faction = boss.Faction;
                        proj.CollisionEff = new CollisionEffect("thunderboss" + "collide", 500);
                        proj.SceneNode = Storage.Scenes["thundercloud"].copy();
                        proj.SceneNode.Position = boss.Position + new Vector2(1, -200);
                        if (proj.SceneNode.containsTexture("start"))//this is how the animation shows itself in its separate stages
                        {
                            proj.SceneNode.playOnce("start", "mid");
                        }
                        BaalaAgni.getInstance().Model.Region.addEntity(proj);
                    }
                    else if (!bossTimer.IsRunning)
                    {
                        bossTimer.Start();
                    }
                    break;

            }
        }

        public override string ToString()
        {
            StringBuilder s = new StringBuilder();
            s.AppendLine("SkillName: " + skillName);
            s.AppendLine("Strength: " + power);
            return s.ToString();

        }

    }
}
