﻿//Matt Monasch: mpm1052@rit.edu
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using BaalaAgni.Collisions;
using BaalaAgni.Effects;

namespace BaalaAgni
{
    public abstract class Entity : EntityObservable, Collideable, Updateable
    {
        private TwoDSceneNode sceneNode;//This will hold the character's position and textures.
        public TwoDSceneNode SceneNode
        {
            get { return sceneNode; }
            set { sceneNode = value; }
        }

        private GraphRegion graphReg;//GraphRegions are going to be used for pathfinding
        public GraphRegion GraphReg
        {
            get { return graphReg; }
            set { graphReg = value; }
        }

        public Vector2 Position//SceneNode controls the position of the character, so change the position there through this property
        {
            get { return sceneNode.Position; }
            set
            {
                sceneNode.Position = value;
                //position = value;
            }
        }
        #region Collideable Implementaion
        
        /// <summary>
        /// Returns the collisionmap of the entity through
        /// the scene node so that it corresponds to the correct frame
        /// of animation.
        /// </summary>
        public CollisionMap CollisionMap
        {
            get { return SceneNode.CMap;}
        }

        /// <summary>
        /// Entities should always have a pixel map.
        /// </summary>
        public bool RectangleOnly
        {
            get { return false; }
        }

        /// <summary>
        /// Entities are always receptive to a collision from something else.
        /// </summary>
        public bool Receptive
        {
            get { return true; }
        }

        #endregion

        private CollisionEffect collisionEff;//The effect that will be called on the object that collides with this one
        public CollisionEffect CollisionEff
        {
            get { return collisionEff; }
            set { collisionEff = value; }
        }

        private bool isMoveable;//Certain things, like buildings and certain NPCs, should not be moveable
        public bool IsMoveable
        {
            get { return isMoveable; }
            set { isMoveable = value; }
        }

        private Vector2 forwardDirection;//To know which direction to move in, we need to know which way it is facing.
        public Vector2 ForwardDirection
        {
            get { return forwardDirection; }
            set { forwardDirection = value; }
        }

        private bool isActive;//isActive is be true when the entity is within the camera and false otherwise. If this is true, it will be drawn.
        public bool IsActive
        {
            get { return isActive; }
            set { isActive = value; }
        }

        private string name;//All Entities need a name
        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        private string typeName;//To distinguish between other types of entities, entities will know their own name in string form
        public string TypeName
        {
            get { return typeName; }
            set { typeName = value; }
        }

        private double health;//All entities have health since all will be killable
        public double Health
        {
            get { return health; }
            set { health = value; }
        }

        private double maxHealth;
        public double MaxHealth
        {
            get { return maxHealth; }
            set { maxHealth = value; }
        }

        private MoveEffect moveEff;//How the entity will move is defined here.
        public MoveEffect MoveEff
        {
            get { return moveEff; }
            set { moveEff = value; }
        }

        private LinkedList<EntityObserver> observerList;//A list of observers for a particular entity.
        public LinkedList<EntityObserver> ObserverList
        {
            get { return observerList; }
            set { observerList = value; }
        }

        private int faction;//Faction is how will we determine if entities take damage when they collide. This will take care of two enemies colliding and not taking damage or possible having an NPC war where they can damage each other.
        public int Faction
        {
            get { return faction; }
            set { faction = value; }
        }

        private int wrathPoints;//Indicates how much the wrath meter should go up by when this entity is killed. This is also used to indicate your score.
        public int WrathPoints
        {
            get { return wrathPoints; }
            set { wrathPoints = value; }
        }

        public Entity()
        {
            name = "entity";
            typeName = "entity";
            isActive = false;
            forwardDirection = new Vector2(0, -1);
            moveEff = new MoveEffect("default", new Vector2(0, 0));
            observerList = new LinkedList<EntityObserver>();
            collisionEff = new CollisionEffect("default", 10);
            isMoveable = true;
            faction = 0;
            wrathPoints = 3;
            

        }
        public virtual void Collide(CollisionResult cr)//Called when two Entities. Each Entity will call its own collide method.
        {
            if (((Entity)cr.GotHit) == BaalaAgni.getInstance().Model.PC)
            {

            }
            if (((Entity)cr.Moved) == BaalaAgni.getInstance().Model.PC)
            {
            }
            if (((Entity)cr.GotHit) is Building)
            {
                if (((Entity)cr.Moved).Faction == 1)
                {
                    collisionEff.performEffect((Entity)cr.Moved, cr);
                    ((Entity)cr.Moved).Position = ((Entity)cr.Moved).Position + ((Entity)cr.Moved).MoveEff.Direction * -1;
                }
                else if (((Entity)cr.Moved).Faction == 2)
                {
                    ((Entity)cr.Moved).Position = ((Entity)cr.Moved).Position + ((Entity)cr.Moved).MoveEff.Direction * -1;
                }
                else if (((Entity)cr.Moved).Faction == 4)
                {
                    CollisionEff.performEffect((Entity)cr.Moved, cr);
                }
            }
            else
            {
                collisionEff.performEffect((Entity)cr.Moved, cr);
            }
        }

        public virtual void kill()
        {
            isActive = false;
#if DEBUG
            Console.WriteLine(Name + " died.");
#endif
            notifyObservers("dead");
            BaalaAgni.getInstance().Model.Score += wrathPoints;
            BaalaAgni.getInstance().Model.increaseWrath(wrathPoints);
            

        }

        public void registerObserver(EntityObserver obs)
        {
            if (!observerList.Contains(obs)) observerList.AddLast(obs);
        }

        public void removeObserver(EntityObserver obs)
        {
            if (observerList.Contains(obs)) observerList.Remove(obs);
        }

        public void notifyObservers(string message)
        {
            LinkedListNode<EntityObserver> obs = observerList.First;

            while (obs != null)
            {
                LinkedListNode<EntityObserver> next = obs.Next;
                obs.Value.update(this, message);

                obs = next;
            }
        }

        public override string ToString()//prevState will not be here because doing so would cause either a ridiculous amount of output, an endless loop, or a crash.
        {
            StringBuilder s = new StringBuilder();
            s.Append("Entity Name: " + name);
            //s.AppendLine("Typename: " + typeName);
            //s.AppendLine("Health: " + health);
            //s.AppendLine("isActive: " + isActive);
            //s.AppendLine("isMoveable: " + isMoveable);
            //s.AppendLine("ForwardDirection: " + forwardDirection.ToString());
            //s.AppendLine("Position: " + Position.ToString());
            /*s.AppendLine("Observers in the ObserverList: ");
            foreach (EntityObserver obs in observerList)
            {
                s.AppendLine(obs.ToString());
            }*/
            return s.ToString();
        }

        public bool Updated
        {
            get;
            set;
        }
        public virtual void Update(GameTime time)
        {
            //create a copy of the CMAP so that we know where it was.
            //prevCMap.PixelMap = this.CMap.PixelMap;
            //prevCMap.Position = this.CMap.Position;
            //prevCMap.Size = this.CMap.Size;
            //prevCMap = this.CMap;
            if (health <= 0)
            {
                if (SceneNode.containsTexture("end") && !SceneNode.PlayingFinal)
                {
                    SceneNode.playFinal("end");
                }
                else if (!SceneNode.containsTexture("end"))
                {
                    kill();
                }
                if (SceneNode.Done)
                {
                    kill();
                }
                else
                {
                    SceneNode.Update((float)time.ElapsedGameTime.TotalSeconds);
                }
            }
            else
            {
                if (!Updated)
                {
                    SceneNode.Update((float)time.ElapsedGameTime.TotalSeconds);
                    AI();


                    Updated = true;
                }
            }
        }

        public bool canCollideWith(Collideable other)
        {
            if (!(other is Entity))
                return false;
            else if (((Entity)other).Faction == this.Faction)
                return false;
            else if (((Entity)other) is Building)
            {
                if (this.faction == 2)
                {
                    //this.Position = this.moveEff.Direction * -1 + this.Position;
                    return true;
                }
                else if (this.faction == 1)
                {
                    //this.Position = this.moveEff.Direction * -1 + this.Position;
                    return true;
                }
            }
            else if (((Entity)other).health <= 0)
                return false;
            return true;
            //TODO: Matt, check for projectile's here, or perhaps override the method in the projectile class as well.
        }

        public void translate(Vector2 vector)
        {
            sceneNode.TranslateTo(vector+Position);//goes through the sceneNode to change the position of the entity
        }

        public virtual void AI()
        {
            //Move enity according to its MoveEffect
            moveEff.performEffect(this);
        }
    }
}
