﻿//Matt Monasch: mpm1052@rit.edu
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using Microsoft.Xna.Framework.Audio;

namespace BaalaAgni
{
    public class Music
    {
        private List<List<SoundEffect>> trackSets;//Contains all of the sets of songs.
        public List<List<SoundEffect>> TrackSets
        {
            get { return trackSets; }
            set { trackSets = value; }
        }

        private List<SoundEffectInstance> currentTrackSet;
        public List<SoundEffectInstance> CurrentTrackSet
        {
            get { return currentTrackSet; }
            set { currentTrackSet = value; }
        }

        private int trackSetPos;//knows which set of tracks are being played
        public int TrackSetPos
        {
            get { return trackSetPos; }
            set { trackSetPos = value; }
        }

        private int trackPos;//knows which track in the set is being played.
        public int TrackPos
        {
            get { return trackPos; }
            set { trackPos = value; }
        }

        /// <summary>
        /// Sounds that play according to what skill was used
        /// skillSounds 1 starts the colliding sounds. 0th position is for the casting
        /// </summary>
        private List<SoundEffect> skillSounds;
        public List<SoundEffect> SkillSounds
        {
            get { return skillSounds; }
            set { skillSounds = value; }
        }

        /// <summary>
        /// An effect that plays when something dies
        /// </summary>
        private SoundEffect onDeath;
        public SoundEffect OnDeath
        {
            get { return onDeath; }
            set { onDeath = value; }
        }

        /// <summary>
        /// An evil laugh for the boss
        /// </summary>
        private SoundEffect evilLaugh;
        public SoundEffect EvilLaugh
        {
            get { return evilLaugh; }
            set { evilLaugh = value; }
        }

        /// <summary>
        /// Music that plays on GameOver
        /// </summary>
        private SoundEffect gameOverTrack;
        public SoundEffect GameOverTrack
        {
            get { return gameOverTrack; }
            set { gameOverTrack = value; }
        }

        private SoundEffectInstance changeFrom;
        private SoundEffectInstance changeTo;//only Music should ever deal with these

        private static Music music;

        private Music(string filename)
        {
            //The music loader will use GetInstance to create a new object and will give Music all of its data
            //right now, this is hardcoded just to get it to work

            trackSets = new List<List<SoundEffect>>();
            currentTrackSet = new List<SoundEffectInstance>();
            skillSounds = new List<SoundEffect>();
            trackSetPos = 0;
            trackPos = 0;

        }

        public static Music GetInstance()
        {
            if (music == null)
            {
                music = new Music("tracklist");
            }
            return music;
        }

        public void Start()
        {
            List<SoundEffect> sounds = trackSets[0];
            for (int x = 0; x < sounds.Count; x++)
            {
                SoundEffect se = sounds[x];
                if (x == 0)
                {
                    currentTrackSet.Add(se.Play(1f, 0.0f, 0.0f, true));
                }
                else
                {
                    currentTrackSet.Add(se.Play(0f, 0.0f, 0.0f, true));
                }
            }
        }

        public void PlaySkillEffect(int skillNum)
        {
            skillSounds[skillNum].Play();
        }

        public void NextTrack()
        {
            Console.WriteLine("In NextTrack");
            if (trackPos + 1 >= currentTrackSet.Count)
            {
                /*if (trackSets.Count-1 < trackSetPos + 1  )
                {
                    Console.WriteLine("muting music");
                    changeFrom.Stop(true);
                    changeTo.Stop(true);
                }
                else
                {*/
                    Console.WriteLine("Going to next list");
                    //changeFrom = currentTrackSet[currentTrackSet.Count - 1];
                    NextTrackSet();
                    changeTo = currentTrackSet[0];
                    
                //}
            }
            else
            {
                changeFrom = currentTrackSet[trackPos];
                Console.WriteLine("Going to next track");
                changeTo = currentTrackSet[trackPos + 1];
                trackPos++;

            }
            
            ThreadStart crossfadingThreadStart = new ThreadStart(Crossfade);
            Thread crossfade = new Thread(new ThreadStart(crossfadingThreadStart));
            crossfade.Start();
            
        }

        public void Crossfade()
        {
            float max = 1;
            for (double x = 3; x < 12; x += .1)
            {
                changeFrom.Volume = (float)(max * (float)Math.Log(12 - x));
                changeTo.Volume = (float)(max * (float)Math.Log(x - 1));
                //Console.WriteLine("ChangeFrom is at " + changeFrom.Volume);
                //Console.WriteLine("ChangeTo is at " + changeTo.Volume);
            }
            //changeFrom = changeTo;

        }

        public void NextTrackSet()
        {
            ((SoundEffectInstance)currentTrackSet[trackPos]).Volume = 0;
            trackPos = 0;
            if (trackSetPos == 0)
            {
                trackSetPos = 1;
            }
            else if (trackSetPos == 1)
            {
                trackSetPos = 2;
            }
            else if (trackSetPos == 2)
            {
                trackSetPos = 1;
            }
            List<SoundEffect> sounds = trackSets[trackSetPos];
            currentTrackSet.Clear();
            for (int x = 0; x < 4; x++)
            {
                SoundEffect se = sounds[x];
                if (x == 0)
                {
                    currentTrackSet.Add(se.Play(1f, 0.0f, 0.0f, true));
                    changeFrom = currentTrackSet[0];
                }
                else
                {
                    currentTrackSet.Add(se.Play(0f, 0.0f, 0.0f, true));
                }
            }
            

        }

        public void PreviousTrack()
        {
            
            if (trackPos - 1 < 0)
            {
                //We should never be going to a previous trackSet, but I will leave this here just in case we decide to do something with it
            }
            else
            {
                changeFrom = currentTrackSet[trackPos];
                changeTo = currentTrackSet[trackPos - 1];
                trackPos--;
            }
            
            //if (trackPos < 0)
            //{
            //    trackPos = 0;
            //}
            ThreadStart crossfadingThreadStart = new ThreadStart(Crossfade);
            Thread crossfade = new Thread(new ThreadStart(crossfadingThreadStart));
            crossfade.Start();
        }

        public void GameOver()
        {
            changeFrom = currentTrackSet[trackPos];
            changeTo = gameOverTrack.Play(0.0f);
            ThreadStart crossfadingThreadStart = new ThreadStart(Crossfade);
            Thread crossfade = new Thread(new ThreadStart(crossfadingThreadStart));
            crossfade.Start();
        }

        public void Update(string message)
        {
            if (message == "wrathup")
            {
                NextTrack();
            }
            if (message == "wrathdown")
            {
                PreviousTrack();
            }
        }



    }
}
